/***************************************************/
/* Game Excelleration Kit vX                       */
/* Copyright (C) 2001 by HourGlass Software NonInc */
/***************************************************/
#ifndef __GEKVECTOR4__
#define __GEKVECTOR4__
/*! 
    \author MoJo
    \file GEKVector4.h
    \brief 4D Vector Class
*/

//! 4D Vector Class
struct GEKVECTOR4
{
public:
	union
	{
		float xyzw[4];
		float rgba[4];
		struct { float x, y, z, w; };
		struct { float r, g, b, a; };
	};

public:
	inline GEKVECTOR4(void);
	inline GEKVECTOR4(const GEKVECTOR2 &kVector);
	inline GEKVECTOR4(const GEKVECTOR3 &kVector);
	inline GEKVECTOR4(const GEKVECTOR4 &kVector);
	inline GEKVECTOR4(float fX, float fY, float fZ, float fW);

	inline float &operator [] (int iAxis);
	inline bool operator < (const GEKVECTOR4 &kVector) const;
	inline bool operator > (const GEKVECTOR4 &kVector) const;

	//@{
	//! Casting operator
	inline operator float * (void);
	inline operator const float * (void) const;
	//@}

	//@{
	//! Assignment operators
	inline GEKVECTOR4 operator = (const GEKVECTOR2 &kVector);
	inline GEKVECTOR4 operator = (const GEKVECTOR3 &kVector);
	inline GEKVECTOR4 operator = (const GEKVECTOR4 &kVector);
	inline operator -= (const GEKVECTOR4 &kVector);
	inline operator += (const GEKVECTOR4 &kVector);
	inline operator /= (float fScalar);
	inline operator *= (float fScalar);
	//@}

	//@{
	//! Binary Operators
	inline GEKVECTOR4 operator - (const GEKVECTOR4 &kVector) const;
	inline GEKVECTOR4 operator + (const GEKVECTOR4 &kVector) const;
	inline GEKVECTOR4 operator - (const GEKVECTOR4 &kVector);
	inline GEKVECTOR4 operator + (const GEKVECTOR4 &kVector);
	inline GEKVECTOR4 operator / (float fScalar) const;
	inline GEKVECTOR4 operator * (float fScalar) const;
	inline GEKVECTOR4 operator / (float fScalar);
	inline GEKVECTOR4 operator * (float fScalar);
	//@}

	inline void SetLength(float fLength);						//!< Sets the length of a vector
	inline float GetLengthSqr(void) const;						//!< Calculates the square length of a vector
	inline float GetLengthSqr(void);							//!< Calculates the square length of a vector
	inline float GetLength(void) const;							//!< Calculates the length of a vector
	inline float GetLength(void);								//!< Calculates the length of a vector

	inline float GetAngle(const GEKVECTOR4 &kVector);		//!< Calculates the angle between two vectors
	inline GEKVECTOR4 Cross(const GEKVECTOR4 &kVector) const;	//!< Calculates the cross product of two vectors
	inline GEKVECTOR4 Cross(const GEKVECTOR4 &kVector);			//!< Calculates the cross product of two vectors
	inline float Distance(const GEKVECTOR4 &kVector) const;		//!< Calculates the distance to a vector
	inline float Distance(const GEKVECTOR4 &kVector);			//!< Calculates the distance to a vector
	inline float Dot(const GEKVECTOR4 &kVector) const;			//!< Calculates the dot product of two vectors
	inline float Dot(const GEKVECTOR4 &kVector);				//!< Calculates the dot product of two vectors

	inline GEKVECTOR4 Lerp(const GEKVECTOR4 &kVector, float fFactor) const;	//!< Calculates the linear interpolation of two vectors
	inline GEKVECTOR4 Lerp(const GEKVECTOR4 &kVector, float fFactor);		//!< Calculates the linear interpolation of two vectors
	inline GEKVECTOR4 Normal(void) const;									//!< Calculates the normal of a vector
	inline GEKVECTOR4 Normal(void);											//!< Calculates the normal of a vector
	void Normalize(void);													//!< Normalizes a vector
};

inline GEKVECTOR4 operator - (const GEKVECTOR4 &kVector);	//!< Negative Operator
inline GEKVECTOR4 operator - (GEKVECTOR4 &kVector);			//!< Negative Operator

#endif