#include "..\GEKXMath.h"

GEKVECTOR3 GEKNormalize(const GEKVECTOR3 &kVector1, const GEKVECTOR3 &kVector2, const GEKVECTOR3 &kVector3)
{
	GEKVECTOR3 kVectorA(kVector2 - kVector1);
	GEKVECTOR3 kVectorB(kVector3 - kVector1);
	GEKVECTOR3 kCross(kVectorA.Cross(kVectorB));
	return kCross.Normal();
}

GEKPLANE::GEKPLANE(const GEKVECTOR3 &kVector1, const GEKVECTOR3 &kVector2, const GEKVECTOR3 &kVector3)
{
	n = GEKNormalize(kVector1, kVector2, kVector3);
	d = -n.Dot(kVector1);
}

float GEKPLANE::Distance(const GEKVECTOR3 &kPoint) const
{
	return (n.Dot(kPoint) + d);
}

bool GEKPLANE::Intersect(const GEKRAY &kRay) const
{
	float fDistanceA = Distance(kRay.m_kOrigin);
	float fDistanceB = Distance(kRay.m_kOrigin + kRay.m_kVector);
	if((fDistanceA > 0.0f)&&(fDistanceB < 0.0f)) return true;
	if((fDistanceA < 0.0f)&&(fDistanceB > 0.0f)) return true;
	return false;
}

GEKVECTOR3 GEKPLANE::Intersection(const GEKRAY &kRay) const
{
	float fNumer = n.Dot(kRay.m_kOrigin);
	float fDenom = n.Dot(kRay.m_kVector);
	float fInter = (d - (fNumer / fDenom));
	return (kRay.m_kOrigin + (kRay.m_kVector * fInter));
}